/*
 * $Header: /home/cvspublic/jakarta-commons/resources/src/java/org/apache/commons/resources/Messages.java,v 1.11 2004/03/27 21:34:10 dgraham Exp $
 * $Revision: 1.11 $
 * $Date: 2004/03/27 21:34:10 $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources;

import java.io.Serializable;
import java.text.MessageFormat;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.resources.impl.ResourceBundleResourcesFactory;

/**
 * <p>Wrapper around any {@link Resources} object that performs message
 * string lookups from the {@link Resources} instance, and parameter
 * replacement via <code>java.text.MessageFormat</code>.  For convenience,
 * the same functionality is also available via static methods that accept
 * a {@link Resources} parameter.</p>
 *
 * <p>Calls to <code>getMessage()</code> variants without a <code>Locale</code>
 * argument are presumed to be requesting a message string in the default
 * <code>Locale</code> for this JVM.</p>
 *
 * <p>When a <code>getString()</code> call to the underlying {@link Resources}
 * instance fails or returns null, a suitable error message String will be
 * returned.</p>
 */
public class Messages implements Serializable {

    /**
     * Commons Logging instance.
     */
    private static final Log log = LogFactory.getLog(Messages.class);
    

    // ----------------------------------------------------------- Constructors


    /**
     * <p>Construct a {@link Messages} instance that wraps the specified
     * {@link Resources} instance.</p>
     *
     * @param resources The {@link Resources} instance from which
     *  message strings are to be retrieved
     */
    public Messages(Resources resources) {

        this.resources = resources;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The {@link Resources} instance that we are wrapping.</p>
     */
    protected Resources resources = null;


    // ------------------------------------------------------------- Properties


    /**
     * <p>Return the {@link Resources} instance that we are wrapping.</p>
     */
    public Resources getResources() {

        return (this.resources);

    }


    // --------------------------------------------------------- Public Methods


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>.</p>
     *
     * @param key Message key to retrieve
     */
    public String getMessage(String key) {

        return (getMessage(resources, key));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     */
    public String getMessage(Locale locale, String key) {

        return (getMessage(resources, locale, key));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
	public String getMessage(String key, Object[] args) {
		return getMessage(resources, key, args);
	}


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
	public String getMessage(Locale locale, String key, Object[] args) {
		return getMessage(resources, locale, key, args);
	}


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
    public String getMessage(String key, Object arg0) {

        return (getMessage(resources, key, arg0));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
    public String getMessage(Locale locale, String key, Object arg0) {

        return (getMessage(resources, locale, key, arg0));

    }


    // ------------------------------------------------------- Static Variables


    /**
     * <p>The {@link org.apache.commons.resources.ResourcesFactory} that will be used by the
     * <code>getMessages()</code> method.</p>
     */
    protected static ResourcesFactory factory = null;


    // --------------------------------------------------------- Static Methods


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     */
    public static String getMessage(Resources resources, String key) {

        return (getMessage(resources, (Locale) null, key));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     */
	public static String getMessage(
		Resources resources,
		Locale locale,
		String key) {

		String message = null;
		try {
			message = resources.getString(key, locale, null);

		} catch (ResourcesException e) {
			log.debug("Failed retrieving message for key: '" + key + "'.", e);
		}

		if (message == null) {
			message = "???" + key + "???";
		}

		return message;
	}


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
	public static String getMessage(
		Resources resources,
		String key,
		Object[] args) {

		return getMessage(resources, (Locale) null, key, args);
	}


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
	public static String getMessage(
		Resources resources,
		Locale locale,
		String key,
		Object[] args) {

		// TODO - Cache MessageFormat instances?
		String message = getMessage(resources, locale, key);
		MessageFormat format = new MessageFormat(message);
		return (format.format(args));
	}


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
    public static String getMessage(Resources resources,
                                    String key, Object arg0) {

        return (getMessage(resources, (Locale) null, key, arg0));

    }

    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
	public static String getMessage(
		Resources resources,
		Locale locale,
		String key,
		Object arg0) {

		return getMessage(resources, locale, key, new Object[] { arg0 });
	}

    /**
     * <p>Convenience factory method to create a {@link Messages} instance
     * that wraps a {@link Resources} instance that contains message resources
     * for the specified properties file.  It is expected that the resources
     * for each package will be in properties files that are nested in the
     * package directory, with names like <code>LocalStrings.properties</code>
     * for the default messages, and names like
     * <code>LocalStrings_en_US.properties</code> for messages localized to
     * a particular <code>Locale</code>.</p>
     *
     * @param name Package + file name of the properties file for which
     *  local message resources are desired (ie. org.apache.commons.resources.LocalStrings).
     */
    public static Messages getMessages(String name) {

        if (factory == null) {
            factory = new ResourceBundleResourcesFactory();
        }
        
        try {
            Resources resources = factory.getResources(name);
            return (new Messages(resources));
            
        } catch (ResourcesException e) {
            return (null);
        }

    }


}
