/*
 * $Header: /home/cvspublic/jakarta-commons/resources/src/java/org/apache/commons/resources/impl/JDBCResources.java,v 1.2 2004/09/07 23:34:46 jmitchell Exp $
 * $Revision: 1.2 $
 * $Date: 2004/09/07 23:34:46 $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.resources.ResourcesException;

/**
 * <p>Concrete implementation of 
 * {@link org.apache.commons.resources.Resources} that wraps a 
 * JDBC database connection and retrieves values for the given 
 * <code>Locale</code> and have name suffixes reflecting the 
 * <code>Locale</code> for which the document's messages apply.
 * For this specific implementation, resources are looked up in 
 * a hierarchy of database values in a manner identical to that 
 * performed by <code>java.util.ResourceBundle.getBundle().</code>.
 * 
 * </p>
 *
 * <p>The base URL passed to our constructor must contain the base name
 * of a properties file that holds the JDBC datasource configuration.</p>
 * 
 * <p>
 * The expected format of the required properties file might look like this:<br>
 * <code><pre>
 * jdbc.connect.driver               = org.gjt.mm.mysql.Driver
 * jdbc.connect.url                  = jdbc:mysql://localhost/resources
 * jdbc.connect.login                = resourcesTest
 * jdbc.connect.password             = resourcesTest
 * 
 * jdbc.sql.db                       = resources
 * jdbc.sql.table                    = resources
 * jdbc.sql.locale.column            = locale
 * jdbc.sql.key.column               = msgKey
 * jdbc.sql.val.column               = val
 * 
 * org.apache.commons.resource.CACHE = true
 * 
 * </pre></code>
 * 
 * </p>
 *
 * @author James Mitchell
 * @version $Revision: 1.2 $ $Date: 2004/09/07 23:34:46 $
 */
public class JDBCResources extends CollectionResourcesBase {

    /**
     * <p>The <code>Log</code> instance for this class.</p>
     */
    private static final Log log = LogFactory.getLog(JDBCResources.class);

    Connection con = null;

    // ----------------------------------------------------------- Constructors

    /**
     * <p>Create a new 
     * {@link org.apache.commons.resources.Resources} instance with the specified
     * logical name and base resource URL.</p>
     *
     * @param name Logical name of the new instance
     * @param base Base URL of the JDBC configuration properties.
     */
    public JDBCResources(String name, String base) {
        super(name, base);
    }


    // ------------------------------------------------------ Protected Methods


    /**
     * <p>Return a <code>Map</code> containing the name-value mappings for
     * the specified base URL and requested <code>Locale</code>, if there
     * are any.  If there are no defined mappings for the specified
     * <code>Locale</code>, return an empty <code>Map</code> instead.</p>
     *
     * <p>Concrete subclasses must override this method to perform the
     * appropriate lookup.  A typical implementation will construct an
     * absolute URL based on the specified base URL and <code>Locale</code>,
     * retrieve the specified resource file (if any), and parse it into
     * a <code>Map</code> structure.</p>
     *
     * <p>Caching of previously retrieved <code>Map</code>s (if any) should
     * be performed by callers of this method.  Therefore, this method should
     * always attempt to retrieve the specified resource and load it
     * appropriately.</p>
     *
     * @param baseUrl Base URL of the resource files for this 
     * {@link org.apache.commons.resources.Resources} instance
     * @param locale <code>Locale</code> for which name-value mappings
     *  are requested
     */
    protected Map getLocaleMap(String baseUrl, Locale locale) {

        if (log.isDebugEnabled()) {
            log.debug("Loading database configuration'" + locale + "' resources from base '" +
                    baseUrl + "'");
        }

        Properties props = new Properties();
        //getLocaleSuffix(locale) + 
        String name = baseUrl + ".properties";
        InputStream stream = null;

        try {

            // Open an input stream to the URL for this locale (if any)
            if (log.isTraceEnabled()) {
                log.trace("Absolute URL is '" + name + "'");
            }
            URL url = new URL(name);
            stream = url.openStream();

            // Parse the input stream and populate the name-value mappings map
            if (log.isTraceEnabled()) {
                log.trace("Parsing input resource");
            }
            props.load(stream);

        } catch (FileNotFoundException e) {

            // Log and swallow this exception
            if (log.isDebugEnabled()) {
                log.debug("No resources for locale '" + locale +
                          "' from base '" + baseUrl + "'");
            }
            props.clear();

        } catch (IOException e) {

            log.warn("IOException loading locale '" + locale +
                     "' from base '" + baseUrl + "'", e);
            props.clear();

        } finally {

            // Close the input stream that was opened earlier
            if (stream != null) {
                try {
                    stream.close();
                } catch (IOException e) {
                    log.error("Error closing stream.", e);
                }
                stream = null;
            }

        }

        // Return the populated (or empty) properties
        Properties properties = new Properties();
		try {
	        properties = loadData(locale, props);
	        
		} catch (InstantiationException e) {
            log.warn("InstantiationException: locale= '" + locale +
                     "' base= '" + baseUrl + "'", e);
        } catch (IllegalAccessException e) {
            log.warn("IllegalAccessException: locale= '" + locale +
                    "' base= '" + baseUrl + "'", e);
        } catch (ClassNotFoundException e) {
            log.warn("Specified Driver not found, make sure it is on " +
            		"the classpath: locale= '" + locale +
                    "' base= '" + baseUrl + "'", e);
        } catch (SQLException e) {
            log.warn("SQLException: locale= '" + locale +
                    "' base= '" + baseUrl + "'", e);
        }
        return properties;

        

    }

    /**
     * @param locale <code>Locale</code> for which name-value mappings
     *  are requested
     * @param connectionProps The connection properties used to instantiate
     * a JDBC connection.
     */
    private Properties loadData(Locale locale, Properties connectionProps) 
    	throws InstantiationException, IllegalAccessException, 
    	ClassNotFoundException, SQLException {
        
        String driver = connectionProps.getProperty("jdbc.connect.driver");
        String url    = connectionProps.getProperty("jdbc.connect.url");
        String user = connectionProps.getProperty("jdbc.connect.login");
        String pass = connectionProps.getProperty("jdbc.connect.password");
        
        String table = connectionProps.getProperty("jdbc.sql.table");
        String localeColumn = connectionProps.getProperty("jdbc.sql.locale.column");
        String keyColumn    = connectionProps.getProperty("jdbc.sql.key.column");
        String valColumn    = connectionProps.getProperty("jdbc.sql.val.column");
        Properties pairs = new Properties();
        
            Class.forName(driver).newInstance();
    		con = DriverManager.getConnection(url, user, pass);
    		
    		String query = "SELECT " + keyColumn + ", " + valColumn + 
    				" FROM " + table + " where " + localeColumn + "= '" + locale + "'";
    		PreparedStatement stmt = con.prepareStatement(query);
    		ResultSet rs = stmt.executeQuery();
    		
    		while (rs.next()) {
    		    pairs.put(rs.getString(keyColumn), rs.getString(valColumn));
    		}
        
        return pairs;
    }


    public void init() throws ResourcesException {
        super.init();
    }
}
